from unittest import TestCase
from unittest.mock import patch, NonCallableMock

from . import get_base_url
from .models import BasePayment
from .. import ProviderVariant, PROVIDER_CACHE

class TestHelpers(TestCase):
    @patch('django.conf.settings.PAYMENT_HOST', new_callable=NonCallableMock)
    def test_text_get_base_url(self, host):
        host.__str__ = lambda x: "example.com/string"
        self.assertEqual(get_base_url(), "https://example.com/string")

    @patch('django.conf.settings.PAYMENT_HOST')
    def test_callable_get_base_url(self, host):
        host.return_value = "example.com/callable"
        self.assertEqual(get_base_url(), "https://example.com/callable")

    def test_model(self):
        BasePayment.load_providers()
        testp = BasePayment()
        for i in BasePayment.list_providers():
            self.assertIsInstance(i, ProviderVariant)
            testp.variant = i.extra["name"]
            self.assertIn(i.extra["name"], PROVIDER_CACHE)



from django.dispatch import Signal
__all__ = ["status_changed"]

# Signal sent whenever status is changed for a Payment. This usually happens
# when a transaction is either accepted or rejected.
status_changed = Signal()



from unittest import TestCase
from web_payments import RedirectNeeded
from web_payments.status import PaymentStatus
from web_payments.testcommon import create_test_payment

from . import DirectPaymentProvider, BankTransferProvider

Payment = create_test_payment()


class TestDirectProvider(TestCase):
    def setUp(self):
        self.payment = Payment()

    def test_confirm_skip(self):
        provider = DirectPaymentProvider(confirm=True)
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)
        with self.assertRaises(RedirectNeeded):
            provider.get_form(self.payment)
        self.assertEqual(self.payment.status, PaymentStatus.CONFIRMED)

    def test_confirm_noskip(self):
        provider = DirectPaymentProvider(skipform=False, confirm=True)
        form = provider.get_form(self.payment)
        self.assertTrue(form.validate())
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)
        with self.assertRaises(RedirectNeeded):
            provider.get_form(self.payment, form.data)
        self.assertEqual(self.payment.status, PaymentStatus.CONFIRMED)

    def test_noconfirm_skip(self):
        provider = DirectPaymentProvider()
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)
        with self.assertRaises(RedirectNeeded):
            provider.get_form(self.payment)
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)

    def test_noconfirm_noskip(self):
        provider = DirectPaymentProvider(skipform=False)
        form = provider.get_form(self.payment)
        self.assertTrue(form.validate())
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)
        with self.assertRaises(RedirectNeeded):
            provider.get_form(self.payment, form.data)
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)

class TestBankTransferProvider(TestCase):
    def setUp(self):
        self.payment = Payment()

    def test_bank_transfer_confirms(self):
        provider = BankTransferProvider(iban="GL5604449876543210", bic="DABAIE2D", confirm=True)
        form = provider.get_form(self.payment)
        self.assertTrue(form.validate())
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)
        with self.assertRaises(RedirectNeeded):
            provider.get_form(self.payment, form.data)
        self.assertEqual(self.payment.status, PaymentStatus.CONFIRMED)

    def test_bank_transfer_confirms_not(self):
        provider = BankTransferProvider("GL5604449876543210", "DABAIE2D", confirm=False)
        form = provider.get_form(self.payment)
        self.assertTrue(form.validate())
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)
        with self.assertRaises(RedirectNeeded):
            provider.get_form(self.payment, form.data)
        self.assertEqual(self.payment.status, PaymentStatus.WAITING)




import os
from web_payments.translation import Translation

translation_path = os.path.join(os.path.dirname(__file__), "locale")

translation = Translation("web_payments_externalpayments.translation", translation_path, domain="web_payments")



from .translation import translation
_ = translation.gettext_lazy

#may better include from web_payments
__all__ = []

class PaymentStatus:
    WAITING = 'waiting'
    PREAUTH = 'preauth'
    CONFIRMED = 'confirmed'
    REJECTED = 'rejected'
    REFUNDED = 'refunded'
    ERROR = 'error'
    INPUT = 'input'

    CHOICES = [
        (WAITING, _('Waiting for confirmation')),
        (PREAUTH, _('Pre-authorized')),
        (CONFIRMED, _('Confirmed')),
        (REJECTED, _('Rejected')),
        (REFUNDED, _('Refunded')),
        (ERROR, _('Error')),
        (INPUT, _('Input'))]


class FraudStatus:
    UNKNOWN = 'unknown'
    ACCEPT = 'accept'
    REJECT = 'reject'
    REVIEW = 'review'

    CHOICES = [
        (UNKNOWN, _('Unknown')),
        (ACCEPT, _('Passed')),
        (REJECT, _('Rejected')),
        (REVIEW, _('Review'))]
